package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.AcceptanceContent;
import digital.laboratory.platform.reagent.service.AcceptanceContentService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (验收内容)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (验收内容) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/acceptance_content" )
@Api(value = "acceptance_content", tags = "(验收内容)管理")
public class AcceptanceContentController {

    private final  AcceptanceContentService acceptanceContentService;

	/**
	 * 通过id查询(验收内容)
	 * @param acceptanceContentId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{acceptanceContentId}" )
	@PreAuthorize("@pms.hasPermission('reagent_acceptance_content_get')" )
	public R<AcceptanceContent> getById(@PathVariable("acceptanceContentId" ) String acceptanceContentId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AcceptanceContent acceptanceContent = acceptanceContentService.getById(acceptanceContentId);
		return R.ok(acceptanceContent);
		//return R.ok(acceptanceContentService.getById(acceptanceContentId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param acceptanceContent (验收内容)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_content_get')" )
    public R<IPage<AcceptanceContent>> getAcceptanceContentPage(Page<AcceptanceContent> page, AcceptanceContent acceptanceContent, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<AcceptanceContent> acceptanceContentSList = acceptanceContentService.page(page, Wrappers.<AcceptanceContent>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(acceptanceContentSList);
//		return R.ok(acceptanceContentService.page(page, Wrappers.query(acceptanceContent)));
    }


    /**
     * 新增(验收内容)
     * @param acceptanceContent (验收内容)
     * @return R
     */
    @ApiOperation(value = "新增(验收内容)", notes = "新增(验收内容)")
    @SysLog("新增(验收内容)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_content_add')" )
    public R<AcceptanceContent> postAddObject(@RequestBody AcceptanceContent acceptanceContent, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        acceptanceContent.setAcceptanceContentId(IdWorker.get32UUID().toUpperCase());
		if (acceptanceContentService.save(acceptanceContent)) {
			return R.ok(acceptanceContent, "对象创建成功");
		}
		else {
			return R.failed(acceptanceContent, "对象创建失败");
		}
    }

    /**
     * 修改(验收内容)
     * @param acceptanceContent (验收内容)
     * @return R
     */
    @ApiOperation(value = "修改(验收内容)", notes = "修改(验收内容)")
    @SysLog("修改(验收内容)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_content_edit')" )
    public R<AcceptanceContent> putUpdateById(@RequestBody AcceptanceContent acceptanceContent, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (acceptanceContentService.updateById(acceptanceContent)) {
			return R.ok(acceptanceContent, "保存对象成功");
		}
		else {
			return R.failed(acceptanceContent, "保存对象失败");
		}
    }

    /**
     * 通过id删除(验收内容)
     * @param acceptanceContentId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(验收内容)", notes = "通过id删除(验收内容)")
    @SysLog("通过id删除(验收内容)" )
    @DeleteMapping("/{acceptanceContentId}" )
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_content_del')" )
    public R<AcceptanceContent> deleteById(@PathVariable String acceptanceContentId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AcceptanceContent oldAcceptanceContent = acceptanceContentService.getById(acceptanceContentId);

		if (acceptanceContentService.removeById(acceptanceContentId)) {
			return R.ok(oldAcceptanceContent, "对象删除成功");
		}
		else {
			return R.failed(oldAcceptanceContent, "对象删除失败");
		}

    }

}
