package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.CatalogueDetails;
import digital.laboratory.platform.reagent.service.CatalogueDetailsService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (采购目录明细)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购目录明细) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/catalogue_details" )
@Api(value = "catalogue_details", tags = "(采购目录明细)管理")
public class CatalogueDetailsController {

    private final  CatalogueDetailsService catalogueDetailsService;

	/**
	 * 通过id查询(采购目录明细)
	 * @param catalogueDetailsId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{catalogueDetailsId}" )
	@PreAuthorize("@pms.hasPermission('reagent_catalogue_details_get')" )
	public R<CatalogueDetails> getById(@PathVariable("catalogueDetailsId" ) String catalogueDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CatalogueDetails catalogueDetails = catalogueDetailsService.getById(catalogueDetailsId);
		return R.ok(catalogueDetails);
		//return R.ok(catalogueDetailsService.getById(catalogueDetailsId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param catalogueDetails (采购目录明细)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_catalogue_details_get')" )
    public R<IPage<CatalogueDetails>> getCatalogueDetailsPage(Page<CatalogueDetails> page, CatalogueDetails catalogueDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<CatalogueDetails> catalogueDetailsSList = catalogueDetailsService.page(page, Wrappers.<CatalogueDetails>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(catalogueDetailsSList);
//		return R.ok(catalogueDetailsService.page(page, Wrappers.query(catalogueDetails)));
    }


    /**
     * 新增(采购目录明细)
     * @param catalogueDetails (采购目录明细)
     * @return R
     */
    @ApiOperation(value = "新增(采购目录明细)", notes = "新增(采购目录明细)")
    @SysLog("新增(采购目录明细)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_catalogue_details_add')" )
    public R<CatalogueDetails> postAddObject(@RequestBody CatalogueDetails catalogueDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        catalogueDetails.setCatalogueDetailsId(IdWorker.get32UUID().toUpperCase());
		if (catalogueDetailsService.save(catalogueDetails)) {
			return R.ok(catalogueDetails, "对象创建成功");
		}
		else {
			return R.failed(catalogueDetails, "对象创建失败");
		}
    }

    /**
     * 修改(采购目录明细)
     * @param catalogueDetails (采购目录明细)
     * @return R
     */
    @ApiOperation(value = "修改(采购目录明细)", notes = "修改(采购目录明细)")
    @SysLog("修改(采购目录明细)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_catalogue_details_edit')" )
    public R<CatalogueDetails> putUpdateById(@RequestBody CatalogueDetails catalogueDetails, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (catalogueDetailsService.updateById(catalogueDetails)) {
			return R.ok(catalogueDetails, "保存对象成功");
		}
		else {
			return R.failed(catalogueDetails, "保存对象失败");
		}
    }

    /**
     * 通过id删除(采购目录明细)
     * @param catalogueDetailsId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(采购目录明细)", notes = "通过id删除(采购目录明细)")
    @SysLog("通过id删除(采购目录明细)" )
    @DeleteMapping("/{catalogueDetailsId}" )
    @PreAuthorize("@pms.hasPermission('reagent_catalogue_details_del')" )
    public R<CatalogueDetails> deleteById(@PathVariable String catalogueDetailsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CatalogueDetails oldCatalogueDetails = catalogueDetailsService.getById(catalogueDetailsId);

		if (catalogueDetailsService.removeById(catalogueDetailsId)) {
			return R.ok(oldCatalogueDetails, "对象删除成功");
		}
		else {
			return R.failed(oldCatalogueDetails, "对象删除失败");
		}

    }

}
