package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.FixedValueReport;
import digital.laboratory.platform.reagent.service.FixedValueReportService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (定值报告)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (定值报告) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/fixed_value_report" )
@Api(value = "fixed_value_report", tags = "(定值报告)管理")
public class FixedValueReportController {

    private final  FixedValueReportService fixedValueReportService;

	/**
	 * 通过id查询(定值报告)
	 * @param fixedValueReportId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{fixedValueReportId}" )
	@PreAuthorize("@pms.hasPermission('reagent_fixed_value_report_get')" )
	public R<FixedValueReport> getById(@PathVariable("fixedValueReportId" ) String fixedValueReportId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        FixedValueReport fixedValueReport = fixedValueReportService.getById(fixedValueReportId);
		return R.ok(fixedValueReport);
		//return R.ok(fixedValueReportService.getById(fixedValueReportId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param fixedValueReport (定值报告)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_fixed_value_report_get')" )
    public R<IPage<FixedValueReport>> getFixedValueReportPage(Page<FixedValueReport> page, FixedValueReport fixedValueReport, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<FixedValueReport> fixedValueReportSList = fixedValueReportService.page(page, Wrappers.<FixedValueReport>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(fixedValueReportSList);
//		return R.ok(fixedValueReportService.page(page, Wrappers.query(fixedValueReport)));
    }


    /**
     * 新增(定值报告)
     * @param fixedValueReport (定值报告)
     * @return R
     */
    @ApiOperation(value = "新增(定值报告)", notes = "新增(定值报告)")
    @SysLog("新增(定值报告)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_fixed_value_report_add')" )
    public R<FixedValueReport> postAddObject(@RequestBody FixedValueReport fixedValueReport, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        fixedValueReport.setFixedValueReportId(IdWorker.get32UUID().toUpperCase());
		if (fixedValueReportService.save(fixedValueReport)) {
			return R.ok(fixedValueReport, "对象创建成功");
		}
		else {
			return R.failed(fixedValueReport, "对象创建失败");
		}
    }

    /**
     * 修改(定值报告)
     * @param fixedValueReport (定值报告)
     * @return R
     */
    @ApiOperation(value = "修改(定值报告)", notes = "修改(定值报告)")
    @SysLog("修改(定值报告)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_fixed_value_report_edit')" )
    public R<FixedValueReport> putUpdateById(@RequestBody FixedValueReport fixedValueReport, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (fixedValueReportService.updateById(fixedValueReport)) {
			return R.ok(fixedValueReport, "保存对象成功");
		}
		else {
			return R.failed(fixedValueReport, "保存对象失败");
		}
    }

    /**
     * 通过id删除(定值报告)
     * @param fixedValueReportId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(定值报告)", notes = "通过id删除(定值报告)")
    @SysLog("通过id删除(定值报告)" )
    @DeleteMapping("/{fixedValueReportId}" )
    @PreAuthorize("@pms.hasPermission('reagent_fixed_value_report_del')" )
    public R<FixedValueReport> deleteById(@PathVariable String fixedValueReportId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        FixedValueReport oldFixedValueReport = fixedValueReportService.getById(fixedValueReportId);

		if (fixedValueReportService.removeById(fixedValueReportId)) {
			return R.ok(oldFixedValueReport, "对象删除成功");
		}
		else {
			return R.failed(oldFixedValueReport, "对象删除失败");
		}

    }

}
