package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.LatticeForm;
import digital.laboratory.platform.reagent.service.LatticeFormService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * 
 *
 * @author Zhang Xiaolong created at  2023-03-22
 * @describe  前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/lattice_form" )
@Api(value = "lattice_form", tags = "管理")
public class LatticeFormController {

    private final  LatticeFormService latticeFormService;

	/**
	 * 通过id查询
	 * @param latticeFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{latticeFormId}" )
	@PreAuthorize("@pms.hasPermission('reagent_lattice_form_get')" )
	public R<LatticeForm> getById(@PathVariable("latticeFormId" ) String latticeFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        LatticeForm latticeForm = latticeFormService.getById(latticeFormId);
		return R.ok(latticeForm);
		//return R.ok(latticeFormService.getById(latticeFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param latticeForm 
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_lattice_form_get')" )
    public R<IPage<LatticeForm>> getLatticeFormPage(Page<LatticeForm> page, LatticeForm latticeForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<LatticeForm> latticeFormSList = latticeFormService.page(page, Wrappers.<LatticeForm>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(latticeFormSList);
//		return R.ok(latticeFormService.page(page, Wrappers.query(latticeForm)));
    }


    /**
     * 新增
     * @param latticeForm 
     * @return R
     */
    @ApiOperation(value = "新增", notes = "新增")
    @SysLog("新增" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_lattice_form_add')" )
    public R<LatticeForm> postAddObject(@RequestBody LatticeForm latticeForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        latticeForm.setLatticeFormId(IdWorker.get32UUID().toUpperCase());
		if (latticeFormService.save(latticeForm)) {
			return R.ok(latticeForm, "对象创建成功");
		}
		else {
			return R.failed(latticeForm, "对象创建失败");
		}
    }

    /**
     * 修改
     * @param latticeForm 
     * @return R
     */
    @ApiOperation(value = "修改", notes = "修改")
    @SysLog("修改" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_lattice_form_edit')" )
    public R<LatticeForm> putUpdateById(@RequestBody LatticeForm latticeForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (latticeFormService.updateById(latticeForm)) {
			return R.ok(latticeForm, "保存对象成功");
		}
		else {
			return R.failed(latticeForm, "保存对象失败");
		}
    }

    /**
     * 通过id删除
     * @param latticeFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除", notes = "通过id删除")
    @SysLog("通过id删除" )
    @DeleteMapping("/{latticeFormId}" )
    @PreAuthorize("@pms.hasPermission('reagent_lattice_form_del')" )
    public R<LatticeForm> deleteById(@PathVariable String latticeFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        LatticeForm oldLatticeForm = latticeFormService.getById(latticeFormId);

		if (latticeFormService.removeById(latticeFormId)) {
			return R.ok(oldLatticeForm, "对象删除成功");
		}
		else {
			return R.failed(oldLatticeForm, "对象删除失败");
		}

    }

}
