package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.PeriodVerificationPlan;
import digital.laboratory.platform.reagent.service.PeriodVerificationPlanService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (标准物质期间核查计划和确认表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质期间核查计划和确认表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/period_verification_plan" )
@Api(value = "period_verification_plan", tags = "(标准物质期间核查计划和确认表)管理")
public class PeriodVerificationPlanController {

    private final  PeriodVerificationPlanService periodVerificationPlanService;

	/**
	 * 通过id查询(标准物质期间核查计划和确认表)
	 * @param periodVerificationPlanId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{periodVerificationPlanId}" )
	@PreAuthorize("@pms.hasPermission('reagent_period_verification_plan_get')" )
	public R<PeriodVerificationPlan> getById(@PathVariable("periodVerificationPlanId" ) String periodVerificationPlanId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PeriodVerificationPlan periodVerificationPlan = periodVerificationPlanService.getById(periodVerificationPlanId);
		return R.ok(periodVerificationPlan);
		//return R.ok(periodVerificationPlanService.getById(periodVerificationPlanId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param periodVerificationPlan (标准物质期间核查计划和确认表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_period_verification_plan_get')" )
    public R<IPage<PeriodVerificationPlan>> getPeriodVerificationPlanPage(Page<PeriodVerificationPlan> page, PeriodVerificationPlan periodVerificationPlan, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<PeriodVerificationPlan> periodVerificationPlanSList = periodVerificationPlanService.page(page, Wrappers.<PeriodVerificationPlan>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(periodVerificationPlanSList);
//		return R.ok(periodVerificationPlanService.page(page, Wrappers.query(periodVerificationPlan)));
    }


    /**
     * 新增(标准物质期间核查计划和确认表)
     * @param periodVerificationPlan (标准物质期间核查计划和确认表)
     * @return R
     */
    @ApiOperation(value = "新增(标准物质期间核查计划和确认表)", notes = "新增(标准物质期间核查计划和确认表)")
    @SysLog("新增(标准物质期间核查计划和确认表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_period_verification_plan_add')" )
    public R<PeriodVerificationPlan> postAddObject(@RequestBody PeriodVerificationPlan periodVerificationPlan, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        periodVerificationPlan.setPeriodVerificationPlanId(IdWorker.get32UUID().toUpperCase());
		if (periodVerificationPlanService.save(periodVerificationPlan)) {
			return R.ok(periodVerificationPlan, "对象创建成功");
		}
		else {
			return R.failed(periodVerificationPlan, "对象创建失败");
		}
    }

    /**
     * 修改(标准物质期间核查计划和确认表)
     * @param periodVerificationPlan (标准物质期间核查计划和确认表)
     * @return R
     */
    @ApiOperation(value = "修改(标准物质期间核查计划和确认表)", notes = "修改(标准物质期间核查计划和确认表)")
    @SysLog("修改(标准物质期间核查计划和确认表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_period_verification_plan_edit')" )
    public R<PeriodVerificationPlan> putUpdateById(@RequestBody PeriodVerificationPlan periodVerificationPlan, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (periodVerificationPlanService.updateById(periodVerificationPlan)) {
			return R.ok(periodVerificationPlan, "保存对象成功");
		}
		else {
			return R.failed(periodVerificationPlan, "保存对象失败");
		}
    }

    /**
     * 通过id删除(标准物质期间核查计划和确认表)
     * @param periodVerificationPlanId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(标准物质期间核查计划和确认表)", notes = "通过id删除(标准物质期间核查计划和确认表)")
    @SysLog("通过id删除(标准物质期间核查计划和确认表)" )
    @DeleteMapping("/{periodVerificationPlanId}" )
    @PreAuthorize("@pms.hasPermission('reagent_period_verification_plan_del')" )
    public R<PeriodVerificationPlan> deleteById(@PathVariable String periodVerificationPlanId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PeriodVerificationPlan oldPeriodVerificationPlan = periodVerificationPlanService.getById(periodVerificationPlanId);

		if (periodVerificationPlanService.removeById(periodVerificationPlanId)) {
			return R.ok(oldPeriodVerificationPlan, "对象删除成功");
		}
		else {
			return R.failed(oldPeriodVerificationPlan, "对象删除失败");
		}

    }

}
