package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.StandardReserveSolution;
import digital.laboratory.platform.reagent.service.StandardReserveSolutionService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (标准储备溶液配制及使用记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准储备溶液配制及使用记录表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/standard_reserve_solution" )
@Api(value = "standard_reserve_solution", tags = "(标准储备溶液配制及使用记录表)管理")
public class StandardReserveSolutionController {

    private final  StandardReserveSolutionService standardReserveSolutionService;

	/**
	 * 通过id查询(标准储备溶液配制及使用记录表)
	 * @param standardReserveSolutionId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{standardReserveSolutionId}" )
	@PreAuthorize("@pms.hasPermission('reagent_standard_reserve_solution_get')" )
	public R<StandardReserveSolution> getById(@PathVariable("standardReserveSolutionId" ) String standardReserveSolutionId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StandardReserveSolution standardReserveSolution = standardReserveSolutionService.getById(standardReserveSolutionId);
		return R.ok(standardReserveSolution);
		//return R.ok(standardReserveSolutionService.getById(standardReserveSolutionId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param standardReserveSolution (标准储备溶液配制及使用记录表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_standard_reserve_solution_get')" )
    public R<IPage<StandardReserveSolution>> getStandardReserveSolutionPage(Page<StandardReserveSolution> page, StandardReserveSolution standardReserveSolution, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<StandardReserveSolution> standardReserveSolutionSList = standardReserveSolutionService.page(page, Wrappers.<StandardReserveSolution>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(standardReserveSolutionSList);
//		return R.ok(standardReserveSolutionService.page(page, Wrappers.query(standardReserveSolution)));
    }


    /**
     * 新增(标准储备溶液配制及使用记录表)
     * @param standardReserveSolution (标准储备溶液配制及使用记录表)
     * @return R
     */
    @ApiOperation(value = "新增(标准储备溶液配制及使用记录表)", notes = "新增(标准储备溶液配制及使用记录表)")
    @SysLog("新增(标准储备溶液配制及使用记录表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_standard_reserve_solution_add')" )
    public R<StandardReserveSolution> postAddObject(@RequestBody StandardReserveSolution standardReserveSolution, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        standardReserveSolution.setStandardReserveSolutionId(IdWorker.get32UUID().toUpperCase());
		if (standardReserveSolutionService.save(standardReserveSolution)) {
			return R.ok(standardReserveSolution, "对象创建成功");
		}
		else {
			return R.failed(standardReserveSolution, "对象创建失败");
		}
    }

    /**
     * 修改(标准储备溶液配制及使用记录表)
     * @param standardReserveSolution (标准储备溶液配制及使用记录表)
     * @return R
     */
    @ApiOperation(value = "修改(标准储备溶液配制及使用记录表)", notes = "修改(标准储备溶液配制及使用记录表)")
    @SysLog("修改(标准储备溶液配制及使用记录表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_standard_reserve_solution_edit')" )
    public R<StandardReserveSolution> putUpdateById(@RequestBody StandardReserveSolution standardReserveSolution, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (standardReserveSolutionService.updateById(standardReserveSolution)) {
			return R.ok(standardReserveSolution, "保存对象成功");
		}
		else {
			return R.failed(standardReserveSolution, "保存对象失败");
		}
    }

    /**
     * 通过id删除(标准储备溶液配制及使用记录表)
     * @param standardReserveSolutionId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(标准储备溶液配制及使用记录表)", notes = "通过id删除(标准储备溶液配制及使用记录表)")
    @SysLog("通过id删除(标准储备溶液配制及使用记录表)" )
    @DeleteMapping("/{standardReserveSolutionId}" )
    @PreAuthorize("@pms.hasPermission('reagent_standard_reserve_solution_del')" )
    public R<StandardReserveSolution> deleteById(@PathVariable String standardReserveSolutionId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StandardReserveSolution oldStandardReserveSolution = standardReserveSolutionService.getById(standardReserveSolutionId);

		if (standardReserveSolutionService.removeById(standardReserveSolutionId)) {
			return R.ok(oldStandardReserveSolution, "对象删除成功");
		}
		else {
			return R.failed(oldStandardReserveSolution, "对象删除失败");
		}

    }

}
