package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.StorageRegistrationForm;
import digital.laboratory.platform.reagent.service.StorageRegistrationFormService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (试剂耗材入库登记表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材入库登记表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/storage_registration_form" )
@Api(value = "storage_registration_form", tags = "(试剂耗材入库登记表)管理")
public class StorageRegistrationFormController {

    private final  StorageRegistrationFormService storageRegistrationFormService;

	/**
	 * 通过id查询(试剂耗材入库登记表)
	 * @param storageRegistrationFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{storageRegistrationFormId}" )
	@PreAuthorize("@pms.hasPermission('reagent_storage_registration_form_get')" )
	public R<StorageRegistrationForm> getById(@PathVariable("storageRegistrationFormId" ) String storageRegistrationFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StorageRegistrationForm storageRegistrationForm = storageRegistrationFormService.getById(storageRegistrationFormId);
		return R.ok(storageRegistrationForm);
		//return R.ok(storageRegistrationFormService.getById(storageRegistrationFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param storageRegistrationForm (试剂耗材入库登记表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_storage_registration_form_get')" )
    public R<IPage<StorageRegistrationForm>> getStorageRegistrationFormPage(Page<StorageRegistrationForm> page, StorageRegistrationForm storageRegistrationForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<StorageRegistrationForm> storageRegistrationFormSList = storageRegistrationFormService.page(page, Wrappers.<StorageRegistrationForm>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(storageRegistrationFormSList);
//		return R.ok(storageRegistrationFormService.page(page, Wrappers.query(storageRegistrationForm)));
    }


    /**
     * 新增(试剂耗材入库登记表)
     * @param storageRegistrationForm (试剂耗材入库登记表)
     * @return R
     */
    @ApiOperation(value = "新增(试剂耗材入库登记表)", notes = "新增(试剂耗材入库登记表)")
    @SysLog("新增(试剂耗材入库登记表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_storage_registration_form_add')" )
    public R<StorageRegistrationForm> postAddObject(@RequestBody StorageRegistrationForm storageRegistrationForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        storageRegistrationForm.setStorageRegistrationFormId(IdWorker.get32UUID().toUpperCase());
		if (storageRegistrationFormService.save(storageRegistrationForm)) {
			return R.ok(storageRegistrationForm, "对象创建成功");
		}
		else {
			return R.failed(storageRegistrationForm, "对象创建失败");
		}
    }

    /**
     * 修改(试剂耗材入库登记表)
     * @param storageRegistrationForm (试剂耗材入库登记表)
     * @return R
     */
    @ApiOperation(value = "修改(试剂耗材入库登记表)", notes = "修改(试剂耗材入库登记表)")
    @SysLog("修改(试剂耗材入库登记表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_storage_registration_form_edit')" )
    public R<StorageRegistrationForm> putUpdateById(@RequestBody StorageRegistrationForm storageRegistrationForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (storageRegistrationFormService.updateById(storageRegistrationForm)) {
			return R.ok(storageRegistrationForm, "保存对象成功");
		}
		else {
			return R.failed(storageRegistrationForm, "保存对象失败");
		}
    }

    /**
     * 通过id删除(试剂耗材入库登记表)
     * @param storageRegistrationFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(试剂耗材入库登记表)", notes = "通过id删除(试剂耗材入库登记表)")
    @SysLog("通过id删除(试剂耗材入库登记表)" )
    @DeleteMapping("/{storageRegistrationFormId}" )
    @PreAuthorize("@pms.hasPermission('reagent_storage_registration_form_del')" )
    public R<StorageRegistrationForm> deleteById(@PathVariable String storageRegistrationFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StorageRegistrationForm oldStorageRegistrationForm = storageRegistrationFormService.getById(storageRegistrationFormId);

		if (storageRegistrationFormService.removeById(storageRegistrationFormId)) {
			return R.ok(oldStorageRegistrationForm, "对象删除成功");
		}
		else {
			return R.failed(oldStorageRegistrationForm, "对象删除失败");
		}

    }

}
