package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.StorageRoomForm;
import digital.laboratory.platform.reagent.service.StorageRoomFormService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * 
 *
 * @author Zhang Xiaolong created at  2023-03-22
 * @describe  前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/storage_room_form" )
@Api(value = "storage_room_form", tags = "管理")
public class StorageRoomFormController {

    private final  StorageRoomFormService storageRoomFormService;

	/**
	 * 通过id查询
	 * @param storageRoomFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{storageRoomFormId}" )
	@PreAuthorize("@pms.hasPermission('reagent_storage_room_form_get')" )
	public R<StorageRoomForm> getById(@PathVariable("storageRoomFormId" ) String storageRoomFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StorageRoomForm storageRoomForm = storageRoomFormService.getById(storageRoomFormId);
		return R.ok(storageRoomForm);
		//return R.ok(storageRoomFormService.getById(storageRoomFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param storageRoomForm 
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_storage_room_form_get')" )
    public R<IPage<StorageRoomForm>> getStorageRoomFormPage(Page<StorageRoomForm> page, StorageRoomForm storageRoomForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<StorageRoomForm> storageRoomFormSList = storageRoomFormService.page(page, Wrappers.<StorageRoomForm>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(storageRoomFormSList);
//		return R.ok(storageRoomFormService.page(page, Wrappers.query(storageRoomForm)));
    }


    /**
     * 新增
     * @param storageRoomForm 
     * @return R
     */
    @ApiOperation(value = "新增", notes = "新增")
    @SysLog("新增" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_storage_room_form_add')" )
    public R<StorageRoomForm> postAddObject(@RequestBody StorageRoomForm storageRoomForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        storageRoomForm.setStorageRoomFormId(IdWorker.get32UUID().toUpperCase());
		if (storageRoomFormService.save(storageRoomForm)) {
			return R.ok(storageRoomForm, "对象创建成功");
		}
		else {
			return R.failed(storageRoomForm, "对象创建失败");
		}
    }

    /**
     * 修改
     * @param storageRoomForm 
     * @return R
     */
    @ApiOperation(value = "修改", notes = "修改")
    @SysLog("修改" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_storage_room_form_edit')" )
    public R<StorageRoomForm> putUpdateById(@RequestBody StorageRoomForm storageRoomForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (storageRoomFormService.updateById(storageRoomForm)) {
			return R.ok(storageRoomForm, "保存对象成功");
		}
		else {
			return R.failed(storageRoomForm, "保存对象失败");
		}
    }

    /**
     * 通过id删除
     * @param storageRoomFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除", notes = "通过id删除")
    @SysLog("通过id删除" )
    @DeleteMapping("/{storageRoomFormId}" )
    @PreAuthorize("@pms.hasPermission('reagent_storage_room_form_del')" )
    public R<StorageRoomForm> deleteById(@PathVariable String storageRoomFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        StorageRoomForm oldStorageRoomForm = storageRoomFormService.getById(storageRoomFormId);

		if (storageRoomFormService.removeById(storageRoomFormId)) {
			return R.ok(oldStorageRoomForm, "对象删除成功");
		}
		else {
			return R.failed(oldStorageRoomForm, "对象删除失败");
		}

    }

}
