package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.SupplierInformationDTO;
import digital.laboratory.platform.reagent.entity.SupplierInformation;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (服务商/供应商信息)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商/供应商信息) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/supplier_information" )
@Api(value = "supplier_information", tags = "(服务商/供应商信息)管理")
public class SupplierInformationController {

    private final  SupplierInformationService supplierInformationService;

	/**
	 * 通过id查询(服务商/供应商信息)
	 * @param supplierInformationId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{supplierInformationId}" )
//	@PreAuthorize("@pms.hasPermission('reagent_supplier_information_get')" )
	public R<SupplierInformation> getById(@PathVariable("supplierInformationId" ) String supplierInformationId, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SupplierInformation supplierInformation = supplierInformationService.getById(supplierInformationId);

		return R.ok(supplierInformation);

	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param supplierInformation (服务商/供应商信息)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_get')" )
    public R<IPage<SupplierInformation>> getSupplierInformationPage(Page<SupplierInformation> page, SupplierInformation supplierInformation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<SupplierInformation> supplierInformationSList = supplierInformationService.page(page, Wrappers.<SupplierInformation>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(supplierInformationSList);
//		return R.ok(supplierInformationService.page(page, Wrappers.query(supplierInformation)));
    }


    /**
     * 新增(服务商/供应商信息)
     * @param supplierInformationDTO (服务商/供应商信息)
     * @return R
     */
    @ApiOperation(value = "新增服务商/供应商信息", notes = "新增服务商/供应商信息")
    @SysLog("新增服务商/供应商信息" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_add')" )
    public R<SupplierInformation> postAddObject(@RequestBody SupplierInformationDTO supplierInformationDTO, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		SupplierInformation supplierInformation = new SupplierInformation();

		if (supplierInformationService.addInfoById(supplierInformationDTO,supplierInformation)) {
			return R.ok(supplierInformation, "保存成功");
		}
		else {
			return R.failed("保存失败");
		}
    }

    /**
     * 修改(服务商/供应商信息)
     * @param supplierInformationDTO (服务商/供应商信息)
     * @return R
     */
    @ApiOperation(value = "修改服务商/供应商信息", notes = "修改服务商/供应商信息")
    @SysLog("修改服务商/供应商信息" )
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('reagent_supplier_information_edit')" )
    public R<SupplierInformation> putUpdateById(@RequestBody SupplierInformationDTO supplierInformationDTO, HttpServletRequest theHttpServletRequest) {

		Principal principal = theHttpServletRequest.getUserPrincipal();

		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		SupplierInformation supplierInformation = supplierInformationService.editInfoById(supplierInformationDTO);

		if (supplierInformation!=null) {
			return R.ok(supplierInformation, "修改成功");
		}
		else {
			return R.failed("修改失败");
		}
    }
}
